/*
 * Integrated Technology Express IT8705F/IT8712F hardware monitor chip
 *
 ***************************************************************
 * Before calling these routines, one must call method->Open() *
 * After calling these routines, one must call method->Close() *
 ***************************************************************
 *

it87
         Chip         Temp    Volt    Fan     SMBus   IOport
        it8705         3       8       3       yes     yes
        it8712         3       8       3       yes     yes
        sis950         3       8       3       yes     yes

 *
 * by YRS
 */


#include	<stdio.h>
#include	"sensors.h"

/* external (global) data */
extern int smb_slave;
extern LM_METHODS method_isa, method_smb;
extern int numSMBSlave, canSMBSlave[128];


#define	IT87_ADDR_START		0x50	/*0x50-0x5E*/
#define	IT87_ADDR_END		0x5E

#define	IT87_SMBADDR	0x48
#define	IT87_REGCHIP	0x58
#define	IT87_CHIPID		0x90

/* temp nr=0,1,2; volt nr=0,1,...6; fan nr=0,1,2 */
#define	IT87_TEMP(nr)	(0x29 + (nr))
#define	IT87_VOLT(nr)	(0x20 + (nr))
#define	IT87_FAN(nr)	(0x0D + (nr))
#define	IT87_FANDIV		0x0B

static	int		it87_probe(LM_METHODS *methods);
static	float	it87_temp(LM_METHODS *methods, int no);
static	int		it87_fanrpm(LM_METHODS *methods, int no);
static	float	it87_volt(LM_METHODS *methods, int no);

SENSOR it87 = {
	"Int.Tec.Exp. Chip, IT8705F/IT8712F",
	it87_probe,
	it87_temp,
	it87_volt,
	it87_fanrpm
};


#define IT87_chkRegNum 8

/* Register checked for probing */
static int chkReg[] = {
	0x00, 0x01, 0x02, 0x03,
	0x0A, 0x48, 0x50, 0x51,
	0x20, 0x21, 0x22, 0x23,
	0x24, 0x25, 0x26, 0x27,
	0x28, 0x29, 0x2A, 0x2B,
	0x0B, 0x0D, 0x0E, 0x0F,
	-1 };


/*
 *  return 0 if not probed
 */
static	int     it87_probe(LM_METHODS *method)
{
	int save;

	if (method != &method_isa  && method != &method_smb)
		return 0;

	save = smb_slave;

	if (method == &method_smb) {
		if (!set_smb_slave(IT87_ADDR_START, IT87_ADDR_END, &smb_slave))
			goto ret0;

		if (smb_slave != 2 * method->Read(IT87_SMBADDR))
			goto ret0;
	}

	if (chkReg_Probe("Probing ITE7805/7812 chip:\n",
			chkReg, method) < IT87_chkRegNum ||
			(method->Read(IT87_REGCHIP) & 0xFF) != IT87_CHIPID)
		goto ret0;

	return 1;
ret0:
	smb_slave = save;
	return 0;
}


/*
 *	\retval	0xFFFF	no sensor
 *	\retval	other	temperature
 *  no = 0,1,2,...  
 */
static	float	it87_temp( LM_METHODS *method, int no )
{
	if (no < 0 || 3 < no)
		return 0xFFFF;

	return (float) method->Read(IT87_TEMP(no));
}


/*
 *	\retval	0x0000FFFF	no sensor
 *  no = 0,1,2,...  
 */
static	float	it87_volt(LM_METHODS *method, int no)
{
	float fac;

	if (no < 0 || 6 < no)
		return 0xFFFF;

	switch (no) {
		case 0:
		case 1:
		case 2:
			fac = 0.016;
			break;
		case 3:
			fac = 0.016 * 1.68;
			break;
		case 4:
			fac = 0.016 * 3.80;
			break;
		case 5:
			fac = - 0.016 * 3.477;
			break;
		case 6:
			fac = - 0.016 * 1.505;
	}

	return (float) method->Read(IT87_VOLT(no)) * fac;
}


/*
	Controlling Fan Divisor: CR = 0x0B.
	Upper 4bits = upper 2bits for fan2, lower 2bits for fan1.

         7       3     0
        +-+-+-+-+-+-+-+-+     xxx = 000,..,111  div1fac = 1,..,128
        |   |y y y|x x x|     yyy = 000,..,111  div2fac = 1,..,128
        +-+-+-+-+-+-+-+-+    initial values: xx=01, yy=01

 */

/*
 *	\retval	0x0000FFFF	no sensor
 *  no = 0,1,2,...  
 */
static	int		it87_fanrpm(LM_METHODS *method, int no)
{
	int	r, n, div[3] = {1,1,1};

	if (no < 0 || 2 < no)
		return 0xFFFF;

	n = method->Read(IT87_FANDIV);
	div[0] = n & 0x07;
	div[1] = (n >> 3) & 0x07;

	r = method->Read(IT87_FAN(no)) & 0xFF;
	if (r == 0xFF) {
		/* change divisor for the sake of next call ! */
		if (no != 2 && div[no] < 7)
			++(div[no]);
		if (no != 2 && div[no] == 7)
			div[no] = 0;
		r = (n & 0x0F) | (div[0] << 4) | (div[1] << 6);
		method->Write(IT87_FANDIV, r);
		return 0xFFFF;
	} else if (r == 0) {
		return 0xFFFF;
	}

	return 1350000 / (r * (1 << div[no]));
}

